/**
 * 
 * <p>
 * This class represents an array that the user can instantiate to hold an object type specified
 * by the generic parameter <code>&lt;E&gt;</code>
 * </p>
 * 
 * <p>
 * This class offers the benefit of not being limited by the initial size definition of the array.
 * When needed, this class will increase the size of the array by <code>DEFAULT_ARRAY_SIZE</code>.
 * </p>
 * @author sameh
 *
 * @param <E>
 */
public class ArrayBasedList<E extends Comparable <? super E>> implements ListInterface<E> {

	@Override
	public boolean isEmpty() {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public void addSorted(E element) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public E replace(E element, int index) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void removeAll() {
		// TODO Auto-generated method stub
		
	}


	/**
	 * This constant is the default size for the array <code>myArray</code>, unless the
	 * user specifies a preferred size. It is also used when increasing the size of 
	 * <code>myArray</code>.
	 */
	private static final int DEFAULT_ARRAY_SIZE = 100;
	
	/**
	 * Private instance variable which is an array of type <code>&lt;E&gt;</code>.
	 */
	private E[] myArray;
	
	/**
	 * Private instance variable that keeps track of how many objects are currently held in
	 * <code>myArray</code>.
	 */
	private int numElements = 0;

	
	/**
	 * 
	 * This is the default constructor for this class. This constructor will 
	 * define the initial array size to be <code>DEFAULT_ARRAY_SIZE</code>
	 * 
	 */
	public ArrayBasedList() {
		myArray = (E[]) new Object[DEFAULT_ARRAY_SIZE];
	}
	
	/**
	 * 
	 * this constructor creates an array of the specified size
	 * 
	 * @param size	the size of the array to start out
	 */
	public ArrayBasedList(int size) {
		if (size > 0) {
			myArray = (E[]) new Object[size];
		} else {
			myArray = (E[]) new Object[DEFAULT_ARRAY_SIZE];
		}
	}
	
	/**
	 * 
	 * This method is used to find the number of elements currently in the array.
	 * 
	 * @return
	 */
	public int size() {
		return this.numElements;
	}
	
	/**
	 * 
	 * @param element
	 */
	public void add(E element) {
		if (this.numElements == this.myArray.length) {
			this.makeMyArrayLarger();
		}
		this.myArray[this.numElements++] = element;
	}

	
	public boolean add(E element, int index) {
		if (index >= 0) {
			if (this.numElements == this.myArray.length) {
				this.makeMyArrayLarger();
			}
			
			if (index == this.numElements) {
				this.myArray[index] = element;
			} else if (index < this.numElements) {
				for ( int i = numElements ; i > index ; i-- ) {
					this.myArray[i] = this.myArray[i-1];
				}
				this.myArray[index] = element;
				this.numElements++;
			}
		}
		return true;
	}

	/**
	 * method to get the element at the specified index
	 * 
	 * @param index this index of the desired element
	 * @return The element at the specified index, <B>null</B> if index is not valid
	 */
	public E get(int index) {
		if ((index >= 0) && (index < this.numElements)) {
			return this.myArray[index];
		}
		return null;
	}

	public boolean remove(int index) {
		
		if ((index >= 0) && (this.numElements > 0) && (index < this.numElements)) {
			
			if (index != this.numElements - 1) {
				for ( int i = index ; i < this.numElements - 1 ; i++ ) {
					this.myArray[i] = this.myArray[i+1];
				}
			}
			
			this.numElements--;
			return true;
		} else {
			return false;
		}
	}
	
	
	private void makeMyArrayLarger() {
		E[] newArray = (E[]) new Object[this.myArray.length + DEFAULT_ARRAY_SIZE];
		for ( int i = 0 ; i < this.myArray.length ; i++ ) {
			newArray[i] = this.myArray[i];
		}
		this.myArray = newArray;
	}


	@Override
	public String toString() {
		String output = "myArray = [";

		for ( int i = 0 ; i < this.numElements ; i++ ) {
			output = output + this.myArray[i];
			if ( i != this.numElements - 1) {
				output = output + ", ";
			}
		}

		output = output + "]";
		
		return output;
	}
	
}
